##
## Erlend april 2014
## Kriging av vannfringskorrelasjonsvariabelen tau for Nord-Trndelag

# Setter arbeidskatalog og tar inn R-pakkene som trengs
setwd( "C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/OK/")
rm(list=ls())
graphics.off()
library("fields")
library(geoR)
library(scatterplot3d)
library(maptools)
library(automap)
options(geoR.messages =F)

##
## 
##

fdata <- list.files("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/dist_corr/Catch_corr/",pattern ="Catch_*.*")
output <- "vario_param_exp.txt"
fout<- file(output, "w")

## Det utfres kriging for hvert referansefelt. Starter derfor for-lkken her.

for (i in 1:length(fdata)){
  graphics.off()
  Next.fdata <- fdata[i]
  corr_catch <- read.geodata(paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/dist_corr/Catch_corr/",Next.fdata,sep=""), header=T)
  #el.border_explor=read.table("C:/Users/Erlend/Documents/NTNU/Prosjektoppgave/kartdata/OK/study_border.txt")
  el.border <- readShapePoly("C:/Users/Erlend/Documents/NTNU/Prosjektoppgave/kartdata/study_basin.shp", proj4string=CRS("+proj=utm +zone=33 +datum=WGS84"))
  el.border1 <- (el.border@polygons[[1]]@Polygons[[1]]@coords)
  #el.border2 <- (el.border@polygons[[2]]@Polygons[[4]]@coords)
  #el.border3 <- (el.border@polygons[[3]]@Polygons[[4]]@coords)
  #el.border4 <- (el.border@polygons[[4]]@Polygons[[4]]@coords)
  
  # Lager til en tabell med koordinatene for NTE-feltene som skal underskes
  pred_grid_corr <- read.table("C:/Users/Erlend/Documents/NTNU/Prosjektoppgave/kartdata/OK/NTE-felt_coords.txt", header =T)
  pred_grid_corr = pred_grid_corr[,2:3]
  
  # Plotter oversikt over korrelasjonsdata for feltet for kontroll
  plot(corr_catch, border=el.border1,lowess = T)
  title(main = paste("Explor_",Next.fdata,sep=""))
  ##
  par(mfrow=c(1,2))
  hist(corr_catch$data, main="Original data", xlab="Catch_corr")
  hist(log(corr_catch$data), main="Transformed data", xlab="log(Catch_corr)")
  
  # Empiriske variogram
  ##
  # Lager til emipiriske variogram med ulik estimator og med bde sky og "bins".
  # Her kan det gjres tilpasninger, slik det str n er 'default'
  cloud1 <- variog(corr_catch, option ="cloud", max.dist ="all", bin.cloud = T)
  cloud2 <- variog(corr_catch, option ="cloud", estimator.type = "modulus", max.dist="all",bin.cloud = T)
  #bin1 <- variog(corr_catch, uvec = seq(0,350000, by=25000))
  bin1=variog(corr_catch, option="bin")
  #bin2 <- variog(corr_catch, uvec = seq(0,350000, by=25000), estimator.type = "modulus")
  bin2=variog(corr_catch, option="bin", estimator.type="modulus")
  par(mfrow = c(2,2))
  plot(cloud1, main = "classical estimator", xlab = "distance (m)")
  plot(cloud2, main = "Modulus estimator", xlab = "distance (m)")
  plot(bin1, main = "classical estimator", xlab = "distance (m)")
  plot(bin2, main = "Modulus estimator", xlab = "distance (m)")
  ##
  # Velger et av variogrammene og plottes sammen med 4-retningers variogram
  # Variogram tar med all data og deler det inn i fire akser. 0 grader er loddrett ser det ut til.
  corr.vario <- variog(corr_catch, option="bin")     
  # variogram in four directions
  vario.4 <- variog4(corr_catch, option="bin")
  par(mfrow = c(1,2))
  plot(corr.vario, xlab = "distance (m)")
  plot(vario.4, xlab = "distance (m)", lwd =2)
  ##
  
  # Automatisk tilpasning av variogram
  # Dette kan gjres for enkelhets skyld. Lar automatisk tilpassing
  # bestemme parametrene i variogram-modellen.
  
  corr_catch.sp <- read.table(paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/dist_corr/Catch_corr/",Next.fdata,sep=""), header=T)
  corr_catch.sp=corr_catch.sp[complete.cases(corr_catch.sp),]  
  coordinates(corr_catch.sp)=c("x_Coord","y_Coord")
  avgm=autofitVariogram(formula = corr ~ 1 ,input_data=corr_catch.sp, model = c("Sph", "Exp", "Gau", "Mat"), kappa=0.5)
  # avgm er automatisk tilpasset variogram for input p parametre
  pred.grid.sp=pred_grid(c(105000, 565000), c(6905000,7485000), by=5000)
  coordinates(pred.grid.sp)=c("Var1","Var2")
  akcm=autoKrige(formula = corr ~ 1 ,input_data=corr_catch.sp, new_data=pred.grid.sp, model = c("Sph", "Exp", "Gau", "Mat"), kappa=0.5)
  plot(akcm)
  # akcm har da gjort en romlig interpolasjon (kriging) og det plottes for kontroll
  # Tilrettelegger for input i modellen:
  model_type=akcm$var_model[2,1]
  sill=akcm$var_model[2,2]
  nugget=akcm$var_model[1,2]
  range=akcm$var_model[2,3]
  
  # Funksjonen likfit lager variogram-modellen tilpasset korrelasjonsdata for feltet
  # Denne br tilpasses best mulig, ogs ved visuell inspeksjon. I dette studiet er
  # den automatiske tilpasningen brukt.
  ml <- likfit(corr_catch, ini.cov.pars = c(sill, range), fix.nugget = FALSE, nugget =  nugget, 
               fix.kappa = TRUE, kappa = 0.5, fix.lambda = TRUE, lambda =  1, fix.psiA = TRUE, psiA = 0, 
               fix.psiR = TRUE, psiR = 1, cov.model ="gaussian", lik.method = "ML", limits = pars.limits(phi=c(4000, 350000)))  
 
  # 'Variogriam envelopes'
  # Plotter to forskjellige 'envelopes' for sammenligning av modellen(siste) med faktiske data
  # Kodene for  tillegge 'lines' for den tilpassede modellen, ml, er gitt.
  # Parametrene m endres etter den modellen for hvert felt.
  env.mc <- variog.mc.env(corr_catch, obj.var = bin1)
  env.model <- variog.model.env(corr_catch, obj.var = bin1, model = ml)
  par(mfrow = c(1,3)) 
  plot(variog(corr_catch, option="bin"))
  #lines.variomodel(cov.model ="exponential", cov.pars = c(0.0272, 200000),max.dist=350000, 
  #nug =0.0178, lwd = 1, col="red")
  
  #lines.variomodel(cov.model ="spherical", cov.pars = c(0.0181, 200000),max.dist=350000, 
  #nug = 0.0172, lwd = 2)
  
  #lines.variomodel(cov.model ="matern", cov.pars = c(0.0944, 200000),max.dist=350000, 
  #nug = 0.0402, kappa = 0.5, lwd = 3,col="green")
  
  #lines.variomodel(cov.model ="gaussian", cov.pars = c(0.0318, 200000),max.dist=350000,  
  #nug = 0.0203, lwd = 4) 
  #legend("topleft",c("exp", "sph", "mat", "gau"), lwd = c(1,2,3,4))
  
  plot(bin1, envelope = env.mc)
  # title( main = paste(Next.fdata))
  plot(bin1, envelope = env.model) 
  
  #lines.variomodel(cov.model ="exponential", cov.pars = c(0.0272, 200000),max.dist=350000, 
  #nug =0.0178, lwd = 1, col="red")
  
  #lines.variomodel(cov.model ="spherical", cov.pars = c(0.0181, 200000),max.dist=350000, 
  #nug = 0.0172, lwd = 2)
  
  #lines.variomodel(cov.model ="matern", cov.pars = c(0.0944, 200000),max.dist=350000, 
  #nug = 0.0402, kappa = 0.5, lwd = 3,col="green")
  
  #lines.variomodel(cov.model ="gaussian", cov.pars = c(0.0318, 200000),max.dist=350000,  
  #nug = 0.0203, lwd = 4) 
  #    

  ## 5. Kriging og kryss-validering
  
  # Kriging etter den tilpassede modellen
  kcm <- krige.control(type = "ok", obj.mod = ml)

  # Kryssvalidering ved kriging: For  sammenligne tau-verdiene med hva som blir
  # predikert av kriging-metoden. Gjres ved  utelate ett og ett data for 
  # predikere ved hjelp av modellen. Den andre, hvor reest=TRUE, endrer parametrene.
  # Plottes for kontroll. Br ikke vre for store residualer.
  xv.ml <- xvalid(corr_catch, model = kcm)
  xvR.ml <- xvalid(corr_catch, model = kcm, reest = TRUE)
  
  par(mfcol = c(5,2), mar = c(3,3,1,0.5), mgp = c(1.5,0.7,0))
  plot(xv.ml)
  plot(xvR.ml)
  
  # Bruker kriging til  estimere punktene til NTE-feltene
  sk.loci <- krige.conv(corr_catch, krige = kcm, loc = pred_grid_corr)
  
  # Kriging for estimering for punkter i hele omrdet med 5 km opplsning
  apply(el.border1, 2, range)
  pred.grid <- pred_grid(c(105000, 565000), c(6905000,7485000), by=5000)
  sk.grid <- krige.conv(corr_catch, krige = kcm, loc = pred.grid)
  
  # Skriver kart med korrelasjonsprediskjonene for hele omrdet
  bmp(paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/OK/Catch_map/",Next.fdata,"_final.bmp",sep=""),width =400, height=500)  
  image(sk.grid, border=el.border1,xlim = c(100000, 570000), ylim= c(6900000,7500000), main = paste("Pred_map_",Next.fdata,sep=""),xlab="X_Coord (m)", ylab="Y_Coord (m)",panel.first = grid(),col=terrain.colors(20, alpha = 1),asp=1)
  contour(sk.grid, add = TRUE,border=el.border1, xlim = c(100000, 570000), ylim= c(6900000,7500000), drawlabels = TRUE) 
  dev.off()
  
  # Skriver plott med sann og predikert tau-verdi for de 34 andre ref-feltene
  jpeg(paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/OK/Catch_map/","xv_",Next.fdata,".jpg",sep=""))
  plot(corr_catch$data, xv.ml$predicted, main = paste("xv_",Next.fdata,sep=""),xlab = "catch_corr", ylab = "predicted_catch_corr")
  #Add 45-degree line to the scatter plot
  abline(0,1)
  dev.off()
  
  # Avslutter med p skrive tabeller med kryssvalideringsresultater, prediksjoner for NTE-posisjoner,
  # og oppsummering av variogram-modellen. 
  # OBS: nrow i out_xv m vre korrekt antall data i studien for hvert felt. Alts m
  # feltene med tau=NA trekkes fra totale antall felt.
  out_xv <- matrix(round(c(corr_catch$coords, corr_catch$data, xv.ml$predicted, xvR.ml$predicted),
                         digits =3),nrow = 35, ncol =5) 
  out_variogram <- matrix(round(c(bin1$u, bin1$v, bin1$n, bin1$sd, bin1$bins.lim[2:(length(bin1$u)+1)]), digits =3), ncol =5)
  
  write.table(out_xv, file= paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/OK/xv_results/","xv_",Next.fdata,".txt",sep=""),quote =FALSE, sep = ",", col.names = c("X-Coord","Y_Coord",
                                                                                                                                                                                  "Data","XV_one_leave_out","XV_reestimate"),row.names=F,qmethod ="double")
  write.table(cbind(pred_grid_corr,sk.loci$predict,sk.loci$krige.var), file = paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/OK/xv_results/","loci_",Next.fdata,".txt",sep=""),quote =FALSE, sep = ",", col.names = c("X-Coord","Y_Coord",
                                                                                                                                                                                                                                      "Predicted","Variance"),row.names=F, qmethod ="double") 
  write.table(out_variogram, file = paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/OK/Npairs/","Npairs_",Next.fdata,".txt",sep=""),quote =FALSE, sep = ",", col.names = c("Sep.Dist.","Variance",
                                                                                                                                                                                          "No.Pairs", "Sd", "Bin.Limits"),row.names=F,qmethod ="double") 
  cat(paste(c(Next.fdata,ml$cov.model,ml$tausq,ml$sigmasq,ml$phi)),file =fout, sep=",")
  cat("\n",file=fout)       
  graphics.off()
  ##
}
close(fout)