## Erlend april 2014
## OcK.r - Script for ko-kriging (OcK=Ordinary co-Kriging)
# =============================================================
##
setwd( "C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/OcK/")

library(gstat)
library(lattice)
library(sp)
library(geoR)
library(maptools)  
library(grDevices)
library(gridExtra)
library(ggplot2)
library(gridExtra)
library(automap)

rm(list=ls())
graphics.off()
##
# 1. Inputt
covar <- "Cr"
model_type <- "auto"
fs_out <- file(paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/OcK/",covar,"_fs_corr.txt",sep=""),"w")
vario_out <- file(paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/OcK/",covar,"_",model_type,"_covar_param.txt",sep=""),"w")

# Starter analysen for hvert av feltene
for(i in seq(1,11,3)){
  
  Next.fdata_t <- paste("tar_catch_",i,sep=""); Next.fdata_cov <- paste("tar_catch_",i+13,sep="")
  gsdata.corr <- read.table(paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/Tau/",Next.fdata_cov,sep=""),sep = ",",
                            header =TRUE) 
  gsdata.corr <- gsdata.corr[!(is.na(gsdata.corr[,4])),] # Ta vekk radene med NA for tau
 
  gsdata.extra <- read.table(paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/Covariables/",Next.fdata_cov,sep=""),sep = "," ,header =TRUE) 
  #gsdata.extra <- gsdata.extra[!(is.na(gsdata.extra[,7])),]
  
  gsdata.corr[gsdata.corr < 0.0] <- 0 
  gsdata.extra[gsdata.extra < 0.0] <- 0 
  
  pred.grid.1 <- pred_grid(c(105000, 565000), c(6905000,7485000), by=5000)
  names(pred.grid.1) <- c("x","y")
  
  coordinates(gsdata.corr) <- c("x", "y")
  coordinates(gsdata.extra) <- c("x","y")
  coordinates(pred.grid.1) <- c("x","y") 
  
  # Kartfil inn for  koordinatfeste studieomrdet
  el.basin <- readShapePoly("C:/Users/Erlend/Documents/NTNU/Prosjektoppgave/kartdata/study_basin.shp", proj4string=CRS("+proj=utm +zone=33 +datum=WGS84)"))
  #el.basin1 <- (el.basin@polygons[[1]]@Polygons[[1]]@coords)/1000
  #el.basin2 <- (el.basin@polygons[[2]]@Polygons[[1]]@coords)/1000
  #el.basin3 <- (el.basin@polygons[[3]]@Polygons[[1]]@coords)/1000
  #el.basin4 <- (el.basin@polygons[[4]]@Polygons[[1]]@coords)/1000
  #el.basin5 <- (el.basin@polygons[[5]]@Polygons[[3]]@coords)/1000
  el.basin1=(el.basin@polygons[[1]]@Polygons[[1]]@coords)
  
  # Study region boundary
  el.border=el.basin1

  pred_grid_corr <- read.table("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/Alle felt.txt", header =T)
  pred_grid_corr = pred_grid_corr[1:13,7:8]
  names(pred_grid_corr)=c("x","y")
  coordinates(pred_grid_corr)=c("x","y")
 
  # 2. Variogram modell for hovedvariabelen tau
  
  v.corr <- variogram(CORR ~ 1, data=gsdata.corr, cutoff = 250000)
  # Estimering av variogrammodell-parametre ved funksjonen likfit.
  # Bruk parametrene og modelltype fra OK.r scriptet som initielle verdier.
  x <- cbind(gsdata.corr@coords, gsdata.corr@data$CORR)
  x <- as.geodata(x)
  ml <- likfit(x, ini.cov.pars = c(0.118, 220000), fix.nugget = FALSE, nugget =  0.009, 
               fix.kappa = FALSE, kappa = 0.5, fix.lambda = TRUE, lambda =  1, fix.psiA = FALSE, psiA = 0, 
               fix.psiR = TRUE, psiR = 1, cov.model = "gaussian", lik.method = "ML", limits = pars.limits(phi=c(0, 200000)))
  
  psill <- ml$sigmasq
  rang <- ml$phi
  nugget <- ml$tausq
  
  # Lager en variogrammodell med parametrene fra likfit.
  m.corr <- vgm(psill,"Gau",rang,nugget)
  plot(v.corr, pl=TRUE, model=m.corr)
  
  # Tilpasser modellparametrene med 'weighted least-squares'
  
  m.corr.f <- fit.variogram(v.corr, m.corr, fit.sills = TRUE, fit.ranges = FALSE)
  plot(v.corr, pl=TRUE, model=m.corr.f)
  ##
  
  ## Ny ordinr kriging med primrvariabelen
  #
  # Punktestimering
  o.k.p <- krige(CORR ~1, locations=gsdata.corr, newdata=gsdata.extra, model=m.corr.f, nmax = 10)
  o.k.p_NTE=krige(CORR ~1, locations=gsdata.corr, newdata=pred_grid_corr, model=m.corr.f, nmax=10)
  # Hele studieomrdet
  o.k <- krige(CORR ~1, locations=gsdata.corr, newdata=pred.grid.1, model=m.corr.f, nmax = 10)
  # Kryssvalidering (leave-one-out)
  o.k.cv <- krige.cv(CORR ~1, gsdata.corr, model = m.corr, nmax = 10, nfold = nrow(gsdata.corr))
  #### 
  # summary statistikk
  # summary(o.k)
  ##
  # Prosessering av ko-variabler. (Tatt ut i eget script. Covariables_corr.r)
  # For valg av ko-variabel som skal inng i co-kriging, velger den med best 'feature space' korrelasjon med tau.
  bmp(paste("C:/Users/Erlend/Documents/NTNU/Prosjektoppgave/kartdata/OcK/",covar,"_",model_type,"_feature_space_oct2013",Next.fdata_t,".bmp",sep=""))
  attach(as.data.frame(gsdata.corr))
  plot(gsdata.corr@data$SimCoeff.Cr ~ gsdata.corr@data$CORR, pch=1, cex=1.2, ylab="SimCoeff.Cr", xlab="CORR",main = paste(Next.fdata_t,sep=""))
  fs_corr <- cor(gsdata.corr@data$SimCoeff.Cr, gsdata.corr@data$CORR, use = "complete")
  fs_fit = lm(gsdata.corr@data$SimCoeff.Cr ~ gsdata.corr@data$CORR)
  abline(fit <- (fs_fit), col ="blue", lwd =2)
  legend("bottomright", bty="n", legend=paste("corr = ", format(fs_corr, digits=2)),cex = 1)
  dev.off()
  # Variogram-modell for den romlige strukturen til ko-variabelen
  v.aux <- variogram(gsdata.extra@data$SimCoeff.Cr ~ 1, gsdata.extra) 
  
  x.aux <- cbind(gsdata.extra@coords,gsdata.extra@data$SimCoeff.Cr)
  x.aux <- as.geodata(x.aux)
  
  # Automatisk tilpassing av variogrammet for ko-variabel og ordinr kriging
  avgm=autofitVariogram(formula = SimCoeff.Cr ~ 1 ,input_data=gsdata.extra, model = c("Sph", "Exp", "Gau", "Mat"), kappa=0.5)
  
  akcm=autoKrige(formula = SimCoeff.Cr ~ 1 ,input_data=gsdata.extra, new_data=pred.grid.1, model = c("Sph", "Exp", "Gau", "Mat"), kappa=0.5)
  #plot(akcm)
  modeltype=akcm$var_model[2,1]
  sill=akcm$var_model[2,2]
  nugget1=akcm$var_model[1,2]
  range=akcm$var_model[2,3]
  modeltype
  
  # Tilpassing p lik mte for kovariabelen som for ordinr kriging av primrvariabel.
  ml.aux <- likfit(x.aux, ini.cov.pars = c(sill, range), fix.nugget = FALSE, nugget =  nugget1, 
                   fix.kappa = FALSE, kappa = 0.5, fix.lambda = TRUE, lambda =  1, fix.psiA = FALSE, psiA = 0, 
                   fix.psiR = TRUE, psiR = 1, cov.model = "gaussian", lik.method = "ML", limits = pars.limits(phi=c(10, 200000)))
  # Inputt for vgm()
  psill.aux <- ml.aux$sigmasq
  rang.aux <- ml.aux$phi
  nugget.aux <- ml.aux$tausq
 
  m.aux <- vgm(psill.aux,"Gau",rang.aux,nugget.aux)
  (m.aux.f <- fit.variogram(v.aux, m.aux))
  plot(v.aux, pl=TRUE, model=m.aux.f)
 
  ## Bygger opp datastruktur for  kunne utfre ko-kriging siden vi n har den romlige strukturen bde for 
  # primr- og kovariabel. Modellen for ko-kriging m tilpasses variogrammene for bde primr- og kovariabelen,
  # men ogs kryssvariogrammene mellom disse. Den greieste mten  gjre dette p er  tilpasse en liner modell
  # med funksjonen fit.lmc hvor alle tre modellene har lik type og 'range', men kan ha ulik 'nugget' og 'sill.'
  
  # Steg  1. Bygger gstat-objekt med 'sample variograms' for primr- og kovariabel.
  (g <- gstat(NULL, id = "CORR", formula = CORR ~ 1, data = gsdata.corr, nmax = 10))
  (g <- gstat(g, id = "Cr", formula = SimCoeff.Cr ~ 1, data = gsdata.extra, nmax = 10))
  
  # Finner og viser variogrammene og kryssvariogrammet for kokriging.
  # Modellen tar utgangspunkt i den tilpassede modellen for primrvariabelen for  sikre en liner modell av koregionalisering
  # og dermed m ha identisk modelltype og range for alle variogrammene.
  (g <- gstat(g, id = "CORR", model = m.corr.f, fill.all = TRUE))
  v.cross <- variogram(g, cutoff = 250000)
  v.cross[v.cross==0]=NA
  v.cross <- v.cross[!(is.na(v.cross[,2])),]

  # fit.lmc-funksjonen lager den endelige modellen som kokrigingen tar utgangspunkt i. Tar inn variogrammene og
  # gstat-objektet med data. Hvis det er problemer med 'singular covariance' under kokriging er det anbefalt
  # med correct.diagonal=1.01. Se veiledning til gstat-pakken.
  (g <- fit.lmc(v.cross, g, fit.lmc=TRUE, correct.diagonal = 1.01))  
  
  # Kokriging og prediksjon av primr- og kovariabelen
  
  o.c.k <- predict.gstat(g, pred.grid.1, nmax = 10) # Ordinr kokriging og prediksjon for hele studieomrdet
  o.c.k.p <- predict.gstat(g, gsdata.extra, nmax = 10) # For sammenligning med ordinr kriging for feltposisjonene
  o.c.k.p_NTE=predict.gstat(g, pred_grid_corr, nmax=10) # Prediksjoner for NTE-feltene
  gridded(o.c.k) = TRUE
  
  #For kun innenfor grensene til feltet:
  ny.grid=pred_grid(c(105000, 565000), c(6905000,7485000), by=5000)
  nt.grid=locations.inside(ny.grid,el.border,as.is=TRUE)
  names(nt.grid) <- c("x","y")
  coordinates(nt.grid) <- c("x","y")
  o.c.k_for_kart=predict.gstat(g, nt.grid, nmax = 10)
  test=over(o.c.k_for_kart["CORR.pred"],o.c.k["CORR.pred"])
  test=cbind(nt.grid, test)
  
  
  # Lager kart med prediksjonsverdiene for hele studieomrdet.
  bmp(paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/test/",covar,"_",model_type,"_OcK_pred_map",Next.fdata_t,".bmp",sep=""),
      width =400, height=500)
  image(test,xlim = c(105000, 565000), ylim= c(6905000, 7485000),xlab="X_Coord (m)", ylab="Y_Coord (m)",panel.first=grid(), col=terrain.colors(20, alpha = 1),asp=1)
  names(test)=c("x","y")
  coordinates(test)=c("x","y")
  gridded(test)=TRUE
  contour(test, add = TRUE, xlim = c(105000, 565000), ylim= c(6905000, 7485000), drawlabels = TRUE, labcex =1) 
  lines(el.basin1, col="black",lwd="2")
  #lines(el.basin2, col="red")
  #lines(el.basin3, col="red")
  #lines(el.basin4, col="red")
  #lines(el.basin5, col="red")
  #grid()
  title(paste("OcK_catch_",Next.fdata_t,sep=""))
  dev.off()
  #
  
  ## Kryssvalidering
  # Kryssvalidering p lik linje med Ordinr kriging. For sammenligning av predikerte og sanne verdier p kjente punkter.
  # Plotter resultatene til mappen
  cv.c <- gstat.cv(g, vdist = 100, nfold = nrow(gsdata.corr))
 
  bmp(paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/OcK/",covar,"_",model_type,"_xv_OcK_",Next.fdata_t,".bmp",sep=""))
  plot(cv.c$observed, (cv.c$observed - cv.c$residual),main=paste(Next.fdata_t,sep=""),xlab="catch_corr", 
       ylab="predicted_catch_corr")
  abline(0,1, col ="blue", lwd =2)
  y <- cv.c$observed; x <- (cv.c$observed - cv.c$residual) 
  fit <- lm(y ~ x)
  dev.off()
  
  # Skriver forskjellige resultater til mappen
  # 1. Feature space korrelasjon oppsummering (valg av ko-variabel)
  cat(paste(c(paste(covar,"_",model_type,"_catch_",i,sep=""), round(fs_corr, digits=2),round(summary(fit)$adj.r.squared, digits=2),
              round((fs_corr)^2, digits=2))),file =fs_out, sep=",")
  cat("\n",file=fs_out) 
  # 2. OK- og OcK-resultater for sammenligning
  ok_ock_out <- file(paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/OcK/",covar,"_",model_type,"_catch_",i,"_OK_OcK_out.txt",sep="")) 
  write.table(c(o.k.p,"with_covariable", o.c.k.p), file = ok_ock_out ,quote =FALSE, row.names =TRUE, 
              col.names = TRUE, qmethod = "escape")
  ok_ock_out_NTE = file(paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/OcK/",covar,"_",model_type,"_catch_",i,"_OK_OcK_out_NTE.txt",sep=""))
  write.table(c(o.k.p_NTE,"with_covariable", o.c.k.p_NTE), file = ok_ock_out_NTE ,quote =FALSE, row.names =TRUE, 
              col.names = TRUE, qmethod = "escape")
  # 3. OcK kryssvalideringsresultater
  OcK_x_vald <- file(paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/OcK/",covar,"_",model_type,"_catch_",i,"_OcK_X_Vald.txt",sep=""))
  write.table(paste(paste("Catch_",i,sep=""),y,x,sep=","),OcK_x_vald ,quote =FALSE, row.names =TRUE,col.names = TRUE, qmethod = "escape")  
  # 4. OcK grid utputt for GIS plotting 
  write.table(c(o.k,"with_covariable",o.c.k), file = paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/OcK/",covar,"_",model_type,"_catch_",i,"_grid_CoK_oct2012.txt", sep =""),
              quote =FALSE, row.names =TRUE, col.names = TRUE, qmethod = "escape")
  # 5. Variogramparametre
  catch <- paste("Catch_",i,sep="")
  cat(paste(c(catch,g$model$CORR$psill,g$model$CORR$range,g$model$SimCoeff$psill,g$model$SimCoeff$range,
              g$model$CORR.SimCoeff$psill,g$model$CORR.SimCoeff$range,round(fs_corr,digits=2),round(summary(fit)$adj.r.squared,digits=2))),
      file =vario_out,sep=",")
  cat("\n",file=vario_out)
  ##
  rm(list=ls())
  graphics.off()
}
close(fs_out)
close(vario_out)
