## Erlend april 2014
##
# dist_corr.r - Script som finner avstand mellom feltene, korrelasjon mellom vfseriene
# og forbereder input for kriging-scriptet. Romlig analyse.

setwd("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/dist_corr/") 
library("fields")
library("Kendall")
library(lattice)
rm(list=ls())
graphics.off()

# Steg 1: Lese inn koordinattabell fra GIS og vannfringsdata fra Qdata.r
koord = read.table("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/Ref-felt_Atttable.txt", header = T)
Q_inn = matrix(scan("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/Q_ref_felt.txt",skip=1),35430,36, byrow=T)
Q = Q_inn[,2:36]

# Kan evt. se p data og plotte hydrogram for  sikre at alt er oK
# view(koord)
# view(Q)
# plot(as.ts(Q[,1]))


# Steg 2: Klargjre utputt fra scriptet.Gir navn p utputt-filer og pne dem for skriving 

output <- "rdist.txt"
fout<- file(output, "w")
output1 <- "corr_out_updated.txt"
fout1 <- file(output1, "w")

# Steg 3: Finne distansen for alle par av referansefelt og skrive til filen definert i steg 2

nr_felt = 35
for (i in 1:nr_felt){
  koord_1 = koord[i,7:8]
  
  for (j in 1:nr_felt){
    koord_2 = koord[j,7:8]
    rdist = dist(rbind(koord_1, koord_2), method="euclidean")
    cat(paste(c(i,j,round(rdist,digit =3))),file =fout, sep=",")
    cat("\n",file=fout)
  }
}
close(fout)

# Steg 4: Finne korrelasjonen mellom vannfringsdataene, alts primrvariabelen
#         Kendall's tau for alle par av referansefelt. Mye data gjr dette tidkrevende.
#         Skriver verdiene til filen definert i steg 2 (fout1 i mitt tilfelle).
#         OBS: For ikke-overlappende dataserier vil utputten bli tau = 1. Dette blir behandlet i steg 5.
nr_felt = 35
for (ii in 1:nr_felt){
  Q_1 <- Q[1:35430,ii] #Fra 1916 til i dag
  
  for (jj in 1:nr_felt){
    Q_2 <- Q[1:35430,jj]
    Q_Kendall <- Kendall(Q_1, Q_2)
    cat(paste(c((ii-1),(jj-1), Q_Kendall)),file =fout1, sep=",")
    cat("\n",file=fout1)
  }
}

close(fout1)

# Steg 5: Kode for  samle de to filene med distanser og korrelasjon i samme fil

fout <- read.table("rdist.txt",sep=","); 
fout1 <- read.table("corr_out_updated.txt", sep = ",");
  # Her m korrelasjonsverdiene manipuleres slik at ikke-overlappende serier fr verdi NA
  tau=fout1[,3]
  tau[tau==1]=NA
  for( i in seq(1,1225,36)){
    tau[i]=1
  }
fout2 <- cbind(fout[,1:3], tau); 
write.table(fout2,"rdist_corr_updated.txt", quote = F, col.names = c("Ref_Catch", "Corr_Catch","Dist.","Corr."),row.names =F,
            sep=",")

# Steg 6: Klargjr strukturen for korrelasjonsdata for kriging for alle felt separat.
#         Plotter korrelasjon mot distanse for alle felt samtidig.

plot_data <-read.table("rdist_corr_updated.txt",header = T, sep =",")
coords <-read.table("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/Ref-felt_Atttable.txt",header = T, sep ="")

par(mfrow = c(2,3),mar = c(3,3,3,2)) # Bestemmer hvordan plottene skal organiseres

fout4 <- file(paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/dist_corr/RSQ.txt"),"w")
fout5 <- file(paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/dist_corr/corr_cluster.txt"),"w") # file for cluster analyis

for(i in seq(1,35,1)){
  gauge.i <- i
  fout3 <- file(paste("C:/Users/Erlend/Documents/NTNU/Masteroppgave/Data/dist_corr/Catch_corr/","Catch_",gauge.i,sep=""),"w")  
  sel <- plot_data$Ref_Catch == gauge.i
  plot_data1 <- plot_data[sel,]     # for plotting
  plot_data2 <- plot_data[sel,4]    # for inputt til kriging
  write.table(cbind(coords[,7:8],plot_data2),fout3,quote = F,
              col.names = c("x_Coord","y_Coord","corr"),row.names =F, sep=" ")
  plot(plot_data1[,3:4], main = paste("Catch_",gauge.i), font.main= 3, cex=2,cex.axis=1.5,cex.main = 1.5,pch=21,
       xlab = "Dist.",ylab ="Corr.")#,xlim=c(0,100),ylim=c(0.5,1))  
  lm_model <- lm(plot_data1[,4]~ plot_data1[,3])
  abline(fit <- (lm_model), col="blue", lwd=2)
  legend("bottomleft", bty="n", legend=paste("R2 = ",
                                             format(summary(fit)$adj.r.squared, digits=2)),cex = 1.5)
  cat(paste(c(paste("catch_",gauge.i,sep=""), round(summary(fit)$adj.r.squared, digits=1))),file =fout4, sep=",")
  cat("\n",file=fout4) 
  cat(paste(c(paste("catch_",gauge.i,sep=""), round(plot_data2, digits=3))),file =fout5, sep=",")
  cat("\n",file=fout5)
  grid()
  close(fout3)
}
close(fout4)
close(fout5)
##